--syn: handles the camera movement in game state
-- local modules
local lg = love.graphics
local flr = math.floor
local min = math.min
local max = math.max
local function lerp(a, b, x) return a + (b - a)*x end
--fields
local dx,dy,tx,ty
local screen_width, screen_height
local map_width_pixels, map_height_pixels
local map_origin_x, map_origin_y
local screen_max_x, screen_max_y
local camera_edge = 24
local camera_speed = 0.008
local camera_speed_modifier
local move_to_x, move_to_y, move_to_timer, move_to_time_allowed
--helper
local function clamp_position()
    --floor the numbers first
    tx = flr(tx)
    ty = flr(ty)
    --clamp
    tx = min(0, tx)
    tx = max(tx, screen_max_x)
    ty = min(0, ty)
    ty = max(ty, screen_max_y)
end
--interface
local init = function(screen_w_, screen_h_, map_w_, map_h_, map_o_x_, map_o_y_)
   dx=0
   dy=0
   tx=0
   ty=0
   screen_width, screen_height = screen_w_, screen_h_
   map_width_pixels = map_w_*TILE_SIZE
   map_height_pixels = map_h_*TILE_SIZE
   map_origin_x, map_origin_y = map_o_x_, map_o_y_
   screen_max_x = screen_width - map_width_pixels - (map_origin_x*2)
   screen_max_y = screen_height- map_height_pixels - (map_origin_y*2)
end

local update = function(dt, mx, my)
    if mx<0 or mx>screen_width or my<0 or my>screen_height then return end

    camera_speed_modifier = 0

    if mx < camera_edge then
        dx = tx + screen_width
        camera_speed_modifier = ((camera_edge - mx)/32)
    elseif mx > screen_width - camera_edge then
        dx = tx - screen_width
        camera_speed_modifier = ((mx - (screen_width - camera_edge))/32)
    else
        dx = nil
    end

    if my < camera_edge then
        dy = ty + screen_height
        camera_speed_modifier = math.min(camera_speed_modifier + (((camera_edge - my)/32)), 1)
    elseif my > screen_height - camera_edge then
        dy = ty - screen_height
        camera_speed_modifier = math.min(camera_speed_modifier + (((my - (screen_height - camera_edge))/32)), 1)
    else
        dy = nil
    end

    if dx then
        tx = lerp(tx, dx, camera_speed * camera_speed_modifier)
        if tx == dx then dx = nil end
    end
    if dy then
        ty = lerp(ty, dy, camera_speed * camera_speed_modifier)
        if ty == dy then dy = nil end
    end

    ------------------------
    -- Move to position
    if move_to_timer then
        tx = lerp(tx, move_to_x, move_to_timer/move_to_time_allowed)
        ty = lerp(ty, move_to_y, move_to_timer/move_to_time_allowed)
        move_to_timer = move_to_timer + 0.01667 --dt
        if move_to_timer>=move_to_time_allowed then
            move_to_x = nil
            move_to_y = nil
            move_to_timer = nil
            move_to_time_allowed = nil
        end
    end

    -- Clamp the position to screen bounds
    clamp_position()
end

local set = function()
    lg.push()
    lg.translate(tx, ty)
end

local unset = function()
    lg.pop()
end

local set_position = function(x,y)
    tx = x
    ty = y
    clamp_position()
end

local get_camera_xy = function()
    return tx, ty
end

local get_camera_xywh = function()
    return tx, ty, screen_width, screen_height
end

local camera = {
  init = init,
  update = update,
  set = set,
  unset = unset,
  get_camera_xy = get_camera_xy,
  get_camera_xywh = get_camera_xywh,
  set_position = set_position,
}
return camera
