local debug_info = require "debug_info"
-- spritebatch glitch test
TILE_SIZE = 32
local lg = love.graphics
local lm = love.mouse
local spritebatch_manager = require 'spritebatch_manager'
local push = require 'push'
local camera = require 'camera'
local spritesheet = require 'spritesheet'
local create_world_map = require 'create_world_map'
local map
local map_width, map_height = 96, 96
local origin_x, origin_y = 48, 48
local font

function love.load()
    math.randomseed(os.time())
    spritesheet.init()
    font = lg.newFont(20)
    lg.setFont(font)
    -- init push module
    local display_divider = 2
    local w, h = lg.getWidth(), lg.getHeight()
    push:setupScreen(w / display_divider, h / display_divider, w, h, {
        fullscreen = true,
        vsync = true,
        resizable = false,
        display_divider = display_divider,
      })
      
      -- Map of game world
      map = create_world_map(map_width, map_height)
      spritebatch_manager.init(map, map_width, map_height, origin_x, origin_y)

      camera.init(push:getWidth(), push:getHeight(), map_width, map_height, origin_x*display_divider, origin_y*display_divider)
end

function love.update(dt)
    spritebatch_manager.update(dt, map)
    local mx, my = push:toGame(lm.getX(), lm.getY())
    camera.update(dt, mx, my)
end

function love.keypressed(key)
    if key == 'escape' then
        love.event.quit()
    end
    if key == 'r' then
        love.load()
    end
    if key == 'p' then
        local temp = os.date("*t")
        lg.captureScreenshot(temp.year..'_'..temp.month..'_'..temp.day..'--('..temp.min..'-'..temp.sec..')--'..'.png')
    end
end

function love.draw()
    -- game state render
    push:start()
    camera.set()

    spritebatch_manager.render()

    camera.unset()
    
    --debug
    local info, w,h = debug_info(spritebatch_manager.get_count(), font)
    lg.setColor(0,0,0,0.75)
    lg.rectangle('fill', 0,0,w,h)
    lg.setColor(1,1,1,1)
    lg.print(info, 0, 0)

    push:finish()

end
