local spritesheet = require 'spritesheet'
local lg = love.graphics
local spritesheet_texture, spritesheet_quads
local map_lower_batch, map_upper_batch, fog_batch, character_batch, character_colour_batch
local map_width, map_height
local origin_x, origin_y

local update_spritebatch = function(batch, map, tile_attribute, colour_attribute)
    assert(batch)
    assert(map)
    batch:clear()
    batch:setColor(1,1,1,1)

    for y=1, map_height do
        for x=1, map_width do
            local this_tile = map[y][x]
            if this_tile[tile_attribute] then
                --check if a colour attribute used
                if colour_attribute then
                    if this_tile[colour_attribute] then
                        batch:setColor(this_tile[colour_attribute])
                    end
                else
                    batch:setColor(1,1,1,1)
                end
                batch:add(spritesheet_quads[this_tile[tile_attribute]], this_tile.x_pos, this_tile.y_pos)
            end
        end
    end

    batch:setColor(1,1,1,1)
    batch:flush()
end

local init = function(map, map_width_, map_height_, origin_x_, origin_y_)
    map_width = map_width_
    map_height = map_height_
    origin_x = origin_x_
    origin_y = origin_y_

    spritesheet_texture, spritesheet_quads = spritesheet.get()

    map_lower_batch = lg.newSpriteBatch(spritesheet_texture, 1024, 'static')
    map_upper_batch = lg.newSpriteBatch(spritesheet_texture, 1024, 'stream')
    character_batch = lg.newSpriteBatch(spritesheet_texture, 1024, 'stream')
    character_colour_batch = lg.newSpriteBatch(spritesheet_texture, 1024, 'stream')
    fog_batch = lg.newSpriteBatch(spritesheet_texture, 1024, 'stream')

    update_spritebatch(map_lower_batch, map, 'map_lower') -- only set once here
    update_spritebatch(map_upper_batch, map, 'map_upper')
    update_spritebatch(character_batch, map, 'character')
    update_spritebatch(character_colour_batch, map, 'character_colour_quad', 'character_colour')
    update_spritebatch(fog_batch, map, 'fog')
end

local update = function(dt, map)
    --update stream batches (in the real game, objects on each of these levels would be
    --changing on a frame by frame basis)
    update_spritebatch(map_upper_batch, map, 'map_upper')
    update_spritebatch(character_batch, map, 'character')
    update_spritebatch(character_colour_batch, map, 'character_colour_quad', 'character_colour')
    update_spritebatch(fog_batch, map, 'fog')
end

 local render = function()
    lg.setColor(1,1,1,1)
    lg.draw(map_lower_batch,origin_x,origin_y)
    lg.draw(map_upper_batch,origin_x,origin_y)
    lg.draw(character_batch,origin_x,origin_y)
    lg.draw(character_colour_batch,origin_x,origin_y)
    lg.draw(fog_batch,origin_x,origin_y)
 end

 local get_count = function()
    local total = 0
    total = total + map_lower_batch:getCount() + map_upper_batch:getCount() + character_batch:getCount() + fog_batch:getCount()
    return total
 end

 local spritebatch_manager = {
    init = init,
    update = update,
    render = render,
    get_count = get_count
 }
 return spritebatch_manager


